@@ -8,6 +8,8 @@ import struct
 
 from PyQt4.QtCore import QCoreApplication, QDir, QFile, QIODevice, QString, \
                          QStringList, QTextCodec
+from ACE_encoding import ACEEncoding
+from ACE_utils    import bytesToInt, intToBytes
 
 dictionaryFile = QDir.homePath() + '/.osso/dictionaries/.personal.dictionary'
 positions = {
@@ -22,47 +24,27 @@ sizes = {
     'header': 8,
     'dictHeader': 8
 }
-codec_overrides = {
-    'ru_RU': 'cp1251',
-}
 
-class ACEFileException(Exception): #{{{1
-    def __init__(self, msgsource, errmsg):
-        self.msgsource = msgsource
-        self.errmsg = errmsg
+class ACEFileReadException(Exception): #{{{1
+    def __init__(self, msg, language, entry = None):
+        self.msg = msg
+        self.language = language
+        self.entry = entry
 
     def __str__(self):
-        return repr("%s: %s" % (self.msgsource, self.errmsg))
+        msg  = 'Error reading dictionary\n\n'
+        msg += '%s for language ID %d' % (self.msg, self.language)
 
-class ACEFileReadException(ACEFileException): #{{{1
-    def __init__(self, errmsg):
-        ACEFileException.__init__(self, 'Error reading dictionary', errmsg)
+        if self.entry != None:
+            msg += '\nLast entry "%s" removed' % self.entry
 
-class ACEFileWriteException(ACEFileException): #{{{1
-    def __init__(self, errmsg):
-        ACEFileException.__init__(self, 'Error writing dictionary', errmsg)
-
-class ACEFileDecodeException(ACEFileException): #{{{1
-    def __init__(self):
-        ACEFileException.__init__(self, 'Error decoding string', 'Encoding tried was %s' % locale.getdefaultlocale()[1])
+        return msg
 
 class ACEFile(): #{{{1
     _dicts = {}
 
     def __init__(self): #{{{2
-        cur_locale = locale.getdefaultlocale()
-
-        if cur_locale[0] in codec_overrides:
-            self._encoding = codec_overrides[cur_locale[0]]
-        else:
-            self._encoding = cur_locale[1]
-
-        self.read()
-
-        return
-    
-    def _bytesToInt(self, bytes): #{{{2
-        return sum(ord(c) << (i * 8) for i, c in enumerate(bytes[::-1]))
+        self._encoding = ACEEncoding()
 
     def _calculateDataSize(self): #{{{2
         size = sizes['header']
@@ -75,43 +57,29 @@ class ACEFile(): #{{{1
         size = 0
 
         for dictEntry in self._dicts[dictLanguage]:
-            size += len(self._fromQString(dictEntry)) + 1
+            size += len(self._encoding.fromQString(dictEntry)) + 1
 
         return size
 
-    def _fromQString(self, strng): #{{{2
-        return unicode(strng.toUtf8(), 'utf-8').encode(self._encoding)
-
-    def _toQString(self, strng): #{{{2
-        try:
-            qStrng = unicode(strng, self._encoding)
-        except Exception, e:
-            raise ACEFileDecodeException
-
-        return qStrng
-
-    def _intToBytes(self, val, count): #{{{2
-        hex = "%%0%dx" % (2 * count) % val
-        bitstream = ''
-
-        for i in xrange(0, count * 2, 2):
-            bitstream += chr(int(hex[i:i+2], 16))
-
-        return bitstream
-
     def _readBufferInt(self, buffer, pos): #{{{2
         posStart = positions[pos][0]
         posEnd   = posStart + positions[pos][1]
 
-        return self._bytesToInt(buffer[posStart:posEnd])
+        return bytesToInt(buffer[posStart:posEnd])
 
-    def _readDictionary(self, file): #{{{2
+    def _readDictionary(self, file, paddingOffset = None): #{{{2
         dictionaryData = file.read(sizes['dictHeader'])
 
         language = self._readBufferInt(dictionaryData, 'dictionary.Language')
         offset = self._readBufferInt(dictionaryData, 'dictionary.StartOffset')
         entryCount = self._readBufferInt(dictionaryData, 'dictionary.EntryCount')
 
+        if paddingOffset == None:
+            dictionaryDataNext = file.read(sizes['dictHeader'])
+            offsetNext = self._readBufferInt(dictionaryDataNext, 'dictionary.StartOffset')
+        else:
+            offsetNext = paddingOffset
+
         self._dicts[language] = QStringList()
         i = 0
 
@@ -122,9 +90,23 @@ class ACEFile(): #{{{1
             self._dicts[language].append(self._readEntry(file))
             i += 1
 
+        if file.pos() < offsetNext:
+            raise ACEFileReadException('Dictionary read short', language)
+        elif file.pos() > offsetNext:
+            lastEntry = self._dicts[language].takeLast()
+
+            if lastEntry.right(1) == self._encoding.toQString('\x00'):
+                while len(lastEntry) > 0 and lastEntry.right(1) == self._encoding.toQString('\x00'):
+                    lastEntry.chop(1)
+
+                raise ACEFileReadException('Dictionary truncated', language, lastEntry)
+            else:
+                self._dicts[language].append(lastEntry)
+                raise ACEFileReadException('Dictionary truncated', language)
+
     def _readEntry(self, file): #{{{2
-        entryLength = self._bytesToInt(file.read(1))
-        entryString = self._toQString(file.read(entryLength))
+        entryLength = bytesToInt(file.read(1))
+        entryString = self._encoding.toQString(file.read(entryLength))
 
         return entryString
 
@@ -143,13 +125,13 @@ class ACEFile(): #{{{1
         posStart = positions[pos][0]
         posEnd = posStart + positions[pos][1]
 
-        newBuffer = buffer[0:posStart] + self._intToBytes(val, positions[pos][1]) + buffer[posEnd:]
+        newBuffer = buffer[0:posStart] + intToBytes(val, positions[pos][1]) + buffer[posEnd:]
 
         return newBuffer
 
     def _writeEntry(self, file, entryString): #{{{2
-        encEntryString = self._fromQString(entryString)
-        file.write(self._intToBytes(len(encEntryString), 1))
+        encEntryString = self._encoding.fromQString(entryString)
+        file.write(intToBytes(len(encEntryString), 1))
         file.write(encEntryString)
 
     def _writeHeader(self, file, dictOrder): #{{{2
@@ -184,6 +166,7 @@ class ACEFile(): #{{{1
         file.write('\x00' * paddingLength)
 
     def read(self): #{{{2
+        self._dicts = {}
         file = QFile(dictionaryFile)
 
         if not file.exists():
@@ -199,12 +182,12 @@ class ACEFile(): #{{{1
             if not file.seek(i * 8):
                 raise IOError(errno.EIO, "Error seeking in file", dictionaryFile)
 
-            self._readDictionary(file)
+            if i == dictionaryCount:
+                self._readDictionary(file, paddingOffset)
+            else:
+                self._readDictionary(file)
             i += 1
 
-        if file.pos() != paddingOffset:
-            raise ACEFileReadException('Error reading dictionary entries')
-
         file.close()
 
     def write(self): #{{{2
@@ -247,6 +230,19 @@ class ACEFile(): #{{{1
     def deleteDict(self, dictLanguage): #{{{2
         del self._dicts[dictLanguage]
 
+    def getEncoding(self): #{{{2
+        return self._encoding.getEncoding()
+
+    def setEncoding(self, enc): #{{{2
+        if enc != self.getEncoding():
+            self._encoding.setEncoding(enc)
+
+            try:
+                self.read()
+            except:
+                self._dicts = {}
+                raise
+
 if __name__ == '__main__': #{{{1
     app = QCoreApplication(sys.argv)
     dict = ACEFile()
