@@ -1,19 +1,25 @@
 #!/usr/bin/env python
 # -*- coding: utf-8 -*-
 
+import errno
+import re
+import sys
+
 from subprocess    import call
 
-from PyQt4.QtCore  import Qt, QObject, QSettings, QString, QStringList, \
-                          QVariant, SIGNAL, SLOT
+from PyQt4.QtCore  import Qt, QObject, QString, QStringList, QVariant, \
+                          SIGNAL, SLOT
 from PyQt4.QtGui   import QAbstractItemView, QComboBox, QDialog, \
                           QDialogButtonBox, QHBoxLayout, QInputDialog, \
                           QLabel, QLineEdit, QListWidget, QMenuBar, \
                           QMessageBox, QPushButton, QStringListModel, \
                           QVBoxLayout, QWidget
-from ACE_file      import ACEFile
+from ACE_file      import ACEFile, ACEFileReadException
 from ACE_utils     import apply_blacklist
+from ACE_settings  import ACESettings
+from ACE_encoding  import ACEEncodingDecodeException
 
-version = '0.0.8'
+version = '0.0.9'
 
 langLookup = {
     3: QString.fromUtf8('English (United Kingdom)'),
@@ -37,51 +43,7 @@ langLookup = {
 }
 
 delModes = QStringList(('Normal', 'Confirm', 'OnClick', 'Blacklist'))
-
-class ACESettings(): #{{{1
-    def __init__(self): #{{{2
-        self._settings = QSettings('Auto-Complete Editor', 'Auto-Complete Editor')
-
-    def getDelMode(self): #{{{2
-        return self._settings.value('DeleteMode').toString()
-
-    def getDisplayDictionary(self): #{{{2
-        return self._settings.value('displayDictionary').toInt()[0]
-
-    def getBlacklist(self): #{{{2
-        blacklist = {}
-
-        self._settings.beginGroup('blacklist')
-        if self._settings.contains('languages'):
-            s = self._settings.value('languages').toString()
-            langList = s.split(',')
-
-            for i in langList:
-                blacklist[int(i)] = self._settings.value(i).toStringList()
-
-        self._settings.endGroup()
-
-        return blacklist
-
-    def setDelMode(self, mode): #{{{2
-        self._settings.setValue('DeleteMode', mode)
-
-    def setDisplayDictionary(self, dict): #{{{2
-        self._settings.setValue('displayDictionary', dict)
-
-    def setBlacklist(self, langList, blacklist): #{{{2
-        self._settings.remove('blacklist')
-
-        if len(langList) == 0:
-            return
-
-        self._settings.beginGroup('blacklist')
-        self._settings.setValue('languages', ','.join(langList))
-
-        for i in langList:
-            self._settings.setValue(i, blacklist[int(i)])
-
-        self._settings.endGroup()
+encodings = QStringList(('Default', 'CP1252', 'ISO8859-1', 'ISO8859-2', 'ISO8859-3', 'ISO8859-4', 'ISO8859-5', 'ISO8859-6', 'ISO8859-7', 'ISO8859-8', 'ISO8859-9', 'ISO8859-10', 'ISO8859-11', 'ISO8859-13', 'ISO8859-14', 'ISO8859-15', 'ISO8859-16', 'UTF-8'))
 
 class ACEAboutWindow(QDialog): #{{{1
     def __init__(self, parent): #{{{2
@@ -115,35 +77,61 @@ class ACEConfigWindow(QDialog): #{{{1
         if parent.maemo:
             from PyQt4.QtMaemo5 import QMaemo5ListPickSelector, QMaemo5ValueButton
 
-            lst = QStringListModel(delModes)
+            lstDelMode = QStringListModel(delModes)
+            lstEncoding = QStringListModel(encodings)
 
-            sel = QMaemo5ListPickSelector()
-            sel.setModel(lst)
-            sel.setCurrentIndex(delModes.indexOf(parent.delMode))
+            selDelMode = QMaemo5ListPickSelector()
+            selDelMode.setModel(lstDelMode)
+            selDelMode.setCurrentIndex(delModes.indexOf(parent.delMode))
 
-            btn = QMaemo5ValueButton('Delete mode')
-            btn.setPickSelector(sel)
+            selEncoding = QMaemo5ListPickSelector()
+            selEncoding.setModel(lstEncoding)
+            selEncoding.setCurrentIndex(encodings.indexOf(parent.encOverride))
 
-            lyt.addWidget(btn)
+            btnDelMode = QMaemo5ValueButton('Delete mode')
+            btnDelMode.setPickSelector(selDelMode)
 
-            QObject.connect(sel, SIGNAL('selected(QString)'), self.changeDeleteMode)
+            btnEncoding = QMaemo5ValueButton('Character encoding')
+            btnEncoding.setPickSelector(selEncoding)
 
-            self._lst = lst
-            self._sel = sel
+            lyt.addWidget(btnDelMode)
+            lyt.addWidget(btnEncoding)
+
+            QObject.connect(selDelMode, SIGNAL('selected(QString)'), self.changeDeleteMode)
+            QObject.connect(selEncoding, SIGNAL('selected(QString)'), self.changeEncoding)
+
+            # If these are not saved, we get a SegFault
+            self._selDelMode  = selDelMode
+            self._selEncoding = selEncoding
+
+            # If these are not saved, the popup lists are empty
+            self._lstDelMode  = lstDelMode
+            self._lstEncoding = lstEncoding
         else:
-            boxLyt = QHBoxLayout()
+            boxLytDelMode = QHBoxLayout()
+            boxLytEncoding = QHBoxLayout()
 
-            lbl = QLabel('Delete mode:')
+            lblDelMode = QLabel('Delete mode:')
+            lblEncoding = QLabel('Character encoding:')
 
-            btn = QComboBox()
-            btn.addItems(delModes)
-            btn.setCurrentIndex(delModes.indexOf(parent.delMode))
+            btnDelMode = QComboBox()
+            btnDelMode.addItems(delModes)
+            btnDelMode.setCurrentIndex(delModes.indexOf(parent.delMode))
 
-            boxLyt.addWidget(lbl)
-            boxLyt.addWidget(btn)
-            lyt.addLayout(boxLyt)
+            btnEncoding = QComboBox()
+            btnEncoding.addItems(encodings)
+            btnEncoding.setCurrentIndex(encodings.indexOf(parent.encOverride))
 
-            QObject.connect(btn, SIGNAL('currentIndexChanged(int)'), self.changeDeleteModeIndex)
+            boxLytDelMode.addWidget(lblDelMode)
+            boxLytDelMode.addWidget(btnDelMode)
+            lyt.addLayout(boxLytDelMode)
+
+            boxLytEncoding.addWidget(lblEncoding)
+            boxLytEncoding.addWidget(btnEncoding)
+            lyt.addLayout(boxLytEncoding)
+
+            QObject.connect(btnDelMode, SIGNAL('currentIndexChanged(int)'), self.changeDeleteModeIndex)
+            QObject.connect(btnEncoding, SIGNAL('currentIndexChanged(int)'), self.changeEncodingIndex)
 
         btns = QDialogButtonBox(QDialogButtonBox.Ok|QDialogButtonBox.Cancel)
         QObject.connect(btns, SIGNAL('accepted()'), self, SLOT('accept()'))
@@ -159,9 +147,19 @@ class ACEConfigWindow(QDialog): #{{{1
         if mode != self._parent.newDelMode:
             self._parent.newDelMode = mode
 
+    def changeEncodingIndex(self, encIndex): #{{{2
+        enc = encodings[encIndex]
+
+        self.changeEncoding(enc)
+
+    def changeEncoding(self, enc): #{{{2
+        if enc != self._parent.encOverride:
+            self._parent.encOverride = enc
+
 class ACEViewWindow(QWidget): #{{{1
     dicts = {}
     maemo = False
+    delMode = 'Normal'
 
     def __init__(self, parent): #{{{2
         QWidget.__init__(self, parent)
@@ -169,6 +167,22 @@ class ACEViewWindow(QWidget): #{{{1
         if self.maemo:
             self.setAttribute(Qt.WA_Maemo5StackedWindow);
 
+    def _applyDeleteMode(self): #{{{2
+        if self.newDelMode != self.delMode:
+            if self.delMode == 'OnClick':
+                self._delBtn.setEnabled(True)
+                QObject.disconnect(self._listView, SIGNAL('itemClicked(QListWidgetItem *)'), self.deleteEntry)
+            elif self.delMode == 'Blacklist':
+                self._delBtn.setText('Delete')
+
+            if self.newDelMode == 'OnClick':
+                self._delBtn.setEnabled(False)
+                QObject.connect(self._listView, SIGNAL('itemClicked(QListWidgetItem *)'), self.deleteEntry)
+            elif self.newDelMode == 'Blacklist':
+                self._delBtn.setText('Blacklist')
+
+            self.delMode = self.newDelMode
+
     def _getLangName(self, langId): #{{{2
         if langId in langLookup:
             return langLookup[langId]
@@ -244,15 +258,10 @@ class ACEViewWindow(QWidget): #{{{1
         self._listView.setSelectionMode(QAbstractItemView.MultiSelection)
         self._listView.setSortingEnabled(True)
 
-        if self.delMode == 'OnClick':
-            QObject.connect(self._listView, SIGNAL('itemClicked(QListWidgetItem *)'), self.deleteEntry)
-
         self._btnBox = QDialogButtonBox()
         self._btnBox.setOrientation(Qt.Vertical)
         btn = self._btnBox.addButton('Delete selected', QDialogButtonBox.ActionRole)
         QObject.connect(btn, SIGNAL('clicked(bool)'), self.deleteEntries)
-        if self.delMode == 'OnClick':
-            btn.setEnabled(False)
         self._delBtn = btn
         btn = self._btnBox.addButton('Add entry', QDialogButtonBox.ActionRole)
         QObject.connect(btn, SIGNAL('clicked(bool)'), self.addEntry)
@@ -264,6 +273,8 @@ class ACEViewWindow(QWidget): #{{{1
         self.layout().addLayout(lyt)
         self.layout().addWidget(self._btnBox)
 
+        self._applyDeleteMode()
+
 class ACEMainWindow(ACEViewWindow): #{{{1
     _dict = None
     blacklists = {}
@@ -289,11 +300,8 @@ class ACEMainWindow(ACEViewWindow): #{{{1
         self.setWindowTitle('Auto-Complete Editor')
 
         self._settings = ACESettings()
-        self.delMode = self._settings.getDelMode()
-        if self.delMode == '':
-            self.delMode = 'Normal'
-
-        self.newDelMode = self.delMode
+        self.newDelMode = self._settings.getDelMode()
+        self.encOverride = self._settings.getEncoding()
 
         QHBoxLayout(self)
 
@@ -316,11 +324,6 @@ class ACEMainWindow(ACEViewWindow): #{{{1
         self._blWindow.dicts = self.blacklists
         self._blWindow.loadData(self._curLang)
         self._blWindow.show()
-#        blacklistWin = ACEBlacklistWindow(self)
-#        blacklistWin.setupScreen()
-#        blacklistWin.loadData(self._curLang)
-
-#        blacklistWin.exec_()
 
         self.saveBlacklists()
 
@@ -330,20 +333,18 @@ class ACEMainWindow(ACEViewWindow): #{{{1
             self.newDelMode = self.delMode
 
         if self.newDelMode != self.delMode:
-            if self.delMode == 'OnClick':
-                self._delBtn.setEnabled(True)
-                QObject.disconnect(self._listView, SIGNAL('itemClicked(QListWidgetItem *)'), self.deleteEntry)
-            elif self.delMode == 'Blacklist':
-                self._delBtn.setText('Delete')
+            self._settings.setDelMode(self.newDelMode);
 
-            if self.newDelMode == 'OnClick':
-                self._delBtn.setEnabled(False)
-                QObject.connect(self._listView, SIGNAL('itemClicked(QListWidgetItem *)'), self.deleteEntry)
-            elif self.newDelMode == 'Blacklist':
-                self._delBtn.setText('Blacklist')
+        self._applyDeleteMode()
 
-            self.delMode = self.newDelMode
-            self._settings.setDelMode(self.delMode);
+        if self.encOverride != self._dict.getEncoding():
+            try:
+                self._dict.setEncoding(self.encOverride)
+                self._settings.setEncoding(self.encOverride)
+            except ACEEncodingDecodeException, e:
+                QMessageBox.critical(self, 'Decode error', 'Unable to decode the dictionary using %s' % self.encOverride)
+            finally:
+                self.loadData()
 
     def loadDictionary(self, row): #{{{2
         ACEViewWindow.loadDictionary(self, row)
@@ -438,6 +439,31 @@ class ACEMainWindow(ACEViewWindow): #{{{1
         if self._dict == None:
             self._dict = ACEFile()
 
+            try:
+                if self.encOverride != 'Default':
+                    self._dict.setEncoding(self.encOverride)
+                else:
+                    self._dict.read()
+            except ACEFileReadException, err:
+                langId = err.language
+                langName = self._getLangName(langId)
+
+                err_msg  = 'Error reading dictionary\n\n'
+                err_msg += '%s for %s' % (err.msg, langName)
+                
+                if err.entry != None:
+                    err_msg += '\nLast entry "%s" removed' % err.entry
+
+                QMessageBox.warning(self, 'Dictionary load error', err_msg)
+            except ACEEncodingDecodeException, err:
+                QMessageBox.critical(self, 'Decode error', 'Unable to decode the dictionary using %s' % self.encOverride)
+            except IOError, err:
+                if err.errno == errno.ENOENT:
+                    QMessageBox.critical(self, 'Dictionary missing', 'Dictionary file not found')
+                    sys.exit()
+                else:
+                    raise
+
         curDict = self._settings.getDisplayDictionary()
         dictLangs = self._dict.getLanguages()
         self._dictList.clear()
@@ -451,11 +477,19 @@ class ACEMainWindow(ACEViewWindow): #{{{1
             self._curLang = curDict
             self._dictList.setCurrentIndex(dictLangs.index(curDict))
         else:
-            self._curLang = dictLangs[0]
-            self._dictList.setCurrentIndex(0)
+            if len(dictLangs) > 0:
+                self._curLang = dictLangs[0]
+                self._dictList.setCurrentIndex(0)
 
-        if len(dictLangs) == 1:
+        if len(dictLangs) == 0:
+            self._btnBox.setEnabled(False)
+        else:
+            self._btnBox.setEnabled(True)
+
+        if len(dictLangs) <= 1:
             self._dictList.setEnabled(False)
+        else:
+            self._dictList.setEnabled(True)
         
         self.blacklists.clear()
 
@@ -474,7 +508,7 @@ class ACEBlacklistWindow(ACEViewWindow): #{{{1
         self.dicts   = parent.blacklists
 
         self.setWindowTitle('Blacklist Editor')
-        self.delMode = 'Normal'
+        self.newDelMode = 'Normal'
 
         QHBoxLayout(self)
 
