@@ -2,12 +2,12 @@
 
 import sys
 
-
 def _cmd_apply_blacklist(): #{{{1
     from PyQt4.QtCore import QCoreApplication
-    from ACE_gui import ACESettings
-    from ACE_file import ACEFile
-    from ACE_utils import apply_blacklist
+    from ACE_settings import ACESettings
+    from ACE_file     import ACEFile
+    from ACE_encoding import ACEEncoding
+    from ACE_utils    import apply_blacklist
 
     app = QCoreApplication(sys.argv)
 
@@ -15,6 +15,8 @@ def _cmd_apply_blacklist(): #{{{1
     blacklists = settings.getBlacklist()
 
     dicts = ACEFile()
+    dicts.setEncoding(settings.getEncoding())
+    dicts.read()
 
     changed = False
     for i in dicts.getLanguages():
@@ -46,7 +48,7 @@ def _cmd_run_gui(): #{{{1
     except Exception, e:
         QMessageBox.critical(None, 'Error', traceback.format_exc(2))
 
-if __name__ == '__main__':
+if __name__ == '__main__': #{{{1
 
     if '--apply-blacklist' in sys.argv:
         _cmd_apply_blacklist()
